@@ -5,6 +5,7 @@
 #include <string.h>
 
 #include "gdmwm.h"
+#include "gdmcommon.h"
 #include "gdmlanguages.h"
 #include "greeter.h"
 #include "greeter_configuration.h"
@@ -85,18 +86,6 @@ greeter_langauge_initialize_model (void)
   g_list_free (list);
 }
 
-void
-greeter_language_init (void)
-{
-  static gboolean initted = FALSE;
-
-  g_assert (initted == FALSE);
-    
-  greeter_langauge_initialize_model ();
-
-  initted = TRUE;
-}
-
 gboolean
 greeter_language_get_save_language (void)
 {
@@ -151,7 +140,7 @@ greeter_language_get_language (const char *old_language)
 	  g_free (current_name);
 	  g_free (saved_name);
 
-	  savelang = greeter_query (msg, TRUE /* markup */, _("Make _Default"), _("Just For _This Session"));
+	  savelang = gdm_common_query (msg, TRUE /* markup */, _("Make _Default"), _("Just For _This Session"));
 	  g_free (msg);
 	}
     }
@@ -174,13 +163,31 @@ selection_changed (GtkTreeSelection *selection,
   gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter, LOCALE_COLUMN, &dialog_selected_language, -1);
 }
 
+static void
+tree_row_activated (GtkTreeView         *view,
+                    GtkTreePath         *path,
+                    GtkTreeViewColumn   *column,
+                    gpointer            data)
+{
+  GtkTreeIter iter;
+  if (gtk_tree_model_get_iter (GTK_TREE_MODEL (lang_model), &iter, path))
+    {
+      g_free (dialog_selected_language);
+      gtk_tree_model_get (GTK_TREE_MODEL (lang_model), &iter,
+			  LOCALE_COLUMN, &dialog_selected_language,
+			  -1);
+      gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+  }
+}
+
 void
 greeter_action_language (GreeterItemInfo *info,
 			 gpointer         user_data)
 {
+  GtkWidget *view = NULL;
+
   if (dialog == NULL)
     {
-      GtkWidget *view;
       GtkWidget *swindow;
       GtkWidget *label;
       char *s;
@@ -208,7 +215,7 @@ greeter_action_language (GreeterItemInfo *info,
       gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
       gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			  label, FALSE, FALSE, 0);
-      view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (lang_model));
+      view = gtk_tree_view_new ();
       gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
       /* FIXME: we should handle this better, but things really look
        * like crap if we aren't always LTR */
@@ -241,10 +248,21 @@ greeter_action_language (GreeterItemInfo *info,
 			"changed",
 			(GCallback) selection_changed,
 			NULL);
+      g_signal_connect(G_OBJECT (view),
+                        "row_activated",
+                        (GCallback) tree_row_activated,
+                        NULL);
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
     }
   gdm_wm_no_login_focus_push ();
+  if (view != NULL)
+    {
+      gtk_widget_show_now (dialog);
+      greeter_langauge_initialize_model ();
+      gtk_tree_view_set_model (GTK_TREE_VIEW (view),
+			       GTK_TREE_MODEL (lang_model));
+    }
   switch (gtk_dialog_run (GTK_DIALOG (dialog)))
     {
     case GTK_RESPONSE_OK:
